@@ -2,7 +2,6 @@ package controller
 
 import (
 	"github.com/openshift/cluster-network-operator/pkg/controller/statusmanager"
-	operatorversion "github.com/openshift/cluster-network-operator/version"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 )
 
@@ -10,11 +9,9 @@ import (
 var AddToManagerFuncs []func(manager.Manager, *statusmanager.StatusManager) error
 
 // AddToManager adds all Controllers to the Manager
-func AddToManager(m manager.Manager) error {
-	s := statusmanager.New(m.GetClient(), m.GetRESTMapper(), "network", operatorversion.Version)
-
+func AddToManager(m manager.Manager, sm *statusmanager.StatusManager) error {
 	for _, f := range AddToManagerFuncs {
-		if err := f(m, s); err != nil {
+		if err := f(m, sm); err != nil {
 			return err
 		}
 	}
